<?php

/*

Plugin Name: First Google Map

Plugin URI: http://designpromote.co.uk/wordpress

Description: Convert a plain text address on Google map using [googlemap] shortcode.

Version: 1.0

Author: Nan Zhang

Author URI: http://designpromote.co.uk

License: GPLv2

*/



define( 'DEFAULT_ADDRESS', 'Edinburgh castle, Edinburgh EH1 2NG');

define( 'DEFAULT_TITLE', 'Edinburgh Castle' );

// Geocode an address: return array of latitude & longitude

function zn_gmap_geocode( $address ) {

    // Make Google Geocoding API URL

    $map_url = 'http://maps.google.com/maps/api/geocode/json?address=';

    $map_url .= urlencode( $address ).'&sensor=false';

    

    // Send GET request

    $request = wp_remote_get( $map_url );



    // Get the JSON object

    $json = wp_remote_retrieve_body( $request );



    // Make sure the request was succesful or return false

    if( empty( $json ) )

        return false;

    

    // Decode the JSON object

    $json = json_decode( $json );



    // Get coordinates

    $lat = $json->results[0]->geometry->location->lat;    //latitude

    $long = $json->results[0]->geometry->location->lng;   //longitude

    

    // Return array of latitude & longitude

    return compact( 'lat', 'long' );

}



// Convert a plain text address into latitude & longitude coordinates

function zn_gmap_get_coords( $address ) {

    

    // Current post id

    global $id;

    

		// Check if this coordinates is in cache

		$cache = wp_cache_get( $id, 'zn_google_maps' );

		if( empty( $cache ) ):

    // Check if we already have this coordinates in the database

    $saved = get_post_meta( $id, 'zn_gmap_addresses' );

    foreach( (array)$saved as $_saved ) {

        if( isset( $_saved['address'] ) && $_saved['address'] == $address ) {

            extract( $_saved );

            return compact( 'lat', 'long' );

        }

    }

    

    // Coordinates not cached: let's fetch them from Google

    $coords = zn_gmap_geocode( $address );

    if( !$coords )

        return false;

    

		$cache = $coords;

		wp_cache_set( $id, $cache, 'zn_google_maps', 60*60*12 );

		

    // Cache result in a post meta data

    add_post_meta( $id, 'zn_gmap_addresses', array(

        'address' => $address,

        'lat' => $coords['lat'],

        'long' => $coords['long']

        )

    );

		endif;

    if( !empty( $cache ) ) $coords = $cache;

		

	// Return array with lat and long

    extract( $coords );

    return compact( 'lat', 'long' );

}

//shortcode in widget (sidebar)

add_action('widget_text', 'do_shortcode');

//add the [google_map] shortcode support

//[googlemap title='business title' width=256 height=256 zoom=14]address[/googlemap]

add_shortcode( 'google_map', 'zn_gmap_generate_map' );

add_shortcode( 'googlemap', 'zn_gmap_generate_map' );

add_shortcode( 'gmap', 'zn_gmap_generate_map' );



// The shortcode callback

function zn_gmap_generate_map( $attr, $address ) {

    // Set map default

    $defaults = array(

				'title' => DEFAULT_TITLE,

        'width'  => '256',

        'height' => '256',

        'zoom'   => 14

    );

    

    // Get map attributes (set to defaults if omitted)

    extract( shortcode_atts( $defaults, $attr ) );

    

    // get coordinates

		if( empty( $address ) ) $address = DEFAULT_ADDRESS;

    $coord = zn_gmap_get_coords( $address );

    

    // Make sure we have coordinates, otherwise return empty string

    if( !$coord )

        return '';

        

    // Output for the shortcode

    $output = '';

        

    // populate $lat and $long variables

    extract( $coord );

    

    // Sanitize variables depending on the context they will be printed in

    $lat     = esc_js( $lat );

    $long    = esc_js( $long );

		$title   = esc_js( $title );

    $address = esc_js( $address );

    $zoom    = esc_js( $zoom );

    $width   = esc_attr( $width );

    $height  = esc_attr( $height );

    

    // generate a unique map ID so we can have different maps on the same page

    $map_id = 'zn_map_'.md5( $address . rand() ); // use rand() to generate #map_id

    

    // Add the Google Maps javascript only once per page

    static $script_added = false;

    if( $script_added == false ) {

        $output .= '<script type="text/javascript"

        src="http://maps.google.com/maps/api/js?sensor=false"></script>';

        $script_added = true;

    }

    

    // Add the map specific code

    $output .= <<<CODE

    <div id="$map_id"></div>

    

    <script type="text/javascript">

    function generate_$map_id() {

			var latlng = new google.maps.LatLng( $lat, $long );

			var options = {

					zoom: $zoom,

					center: latlng,

					mapTypeId: google.maps.MapTypeId.ROADMAP

			}

			

			// var image = new google.maps.MarkerImage( 'http://www.kweilin.net/wp-content/themes/restaurant/images/restaurant.png' );

			// var shadow = new google.maps.MarkerImage( 'http://www.kweilin.net/wp-content/themes/restaurant/images/shadow-big.png' );

			/*var shape = {

				coord: [1, 1, 1, 37, 32, 37, 32 , 1],

				type: 'poly'

			};*/



			var map = new google.maps.Map(

					document.getElementById("$map_id"),

					options

			);



			var legend = '<div class="map_legend"><b>$title</b><br/><p> $address </p></div>';



			var infowindow = new google.maps.InfoWindow ({

					content: legend

			});



			var marker = new google.maps.Marker({

					position: latlng,

					map: map,

					// icon: image,

					// shadow: shadow,

					// shape: shape,

					zIndex: 1

			});

			

			google.maps.event.addListener(marker, 'click', function() {

				infowindow.open(map,marker);

			});

			

			google.maps.event.addListener(map, 'click', function() {

				infowindow.close();

			});

    }



    generate_$map_id();

    

    </script>

    

    <style type="text/css">

    .map_legend{

        width:200px;

        /*max-height:200px;

        min-height:100px;*/

    }

    #$map_id {

        width: {$width}px;

        height: {$height}px;

    }

		#$map_id img {

				max-width: none;

		}

    </style>

    

CODE;



    return $output;

}



?>